@@ -26,7 +26,7 @@ import wandb
 
 import numpy as np
 import litellm  # type: ignore
-from sentence_transformers import SentenceTransformer  # type: ignore
+# sentence-transformers is optional - imported lazily when needed
 from .clustering_prompts import clustering_systems_prompt, coarse_clustering_systems_prompt, deduplication_clustering_systems_prompt
 from stringsight.logging_config import get_logger
 
@@ -51,15 +51,16 @@ _cache_dir, _cache_size = _get_cache_config()
 # Module-level logger (initialize early so we can use it)
 logger = logging.getLogger(__name__)
 
-# Caching is enabled by default, can be disabled via STRINGSIGHT_DISABLE_CACHE
+# Clustering cache is disabled by default, can be enabled via STRINGSIGHT_ENABLE_CLUSTERING_CACHE=1
+# (This is separate from property extraction caching which remains enabled by default)
 import os as _os
-if _os.environ.get("STRINGSIGHT_DISABLE_CACHE", "0") not in ("1", "true", "True"):
+if _os.environ.get("STRINGSIGHT_ENABLE_CLUSTERING_CACHE", "0") in ("1", "true", "True"):
     _cache = Cache(cache_dir=_cache_dir, max_size=_cache_size)
     embedding_status = "disabled" if _os.environ.get("STRINGSIGHT_DISABLE_EMBEDDING_CACHE", "0") in ("1", "true", "True") else "enabled"
     logger.info(f"Caching enabled for clustering (cache_dir={_cache_dir}, embeddings={embedding_status})")
 else:
     _cache = None
-    logger.info("Caching disabled for clustering (set STRINGSIGHT_DISABLE_CACHE=0 to enable)")
+    logger.info("Clustering cache disabled by default (set STRINGSIGHT_ENABLE_CLUSTERING_CACHE=1 to enable)")
 
 # -----------------------------------------------------------------------------
 # Ensure deterministic behaviour in sampling helpers and downstream clustering
@@ -136,11 +137,45 @@ def _get_openai_embeddings_batch(batch: List[str], model: str, retries: int = 3,
         except Exception as exc:
             if attempt == retries - 1:
                 raise
-            logger.warning(f"[retry {attempt + 1}/{retries}] {exc}. Sleeping {sleep_time}s.")
-            time.sleep(sleep_time)
+            
+            # Check if this is a rate limit error
+            is_rate_limit = False
+            retry_after = None
+            
+            # Check for litellm rate limit errors
+            if hasattr(litellm, 'RateLimitError') and isinstance(exc, litellm.RateLimitError):
+                is_rate_limit = True
+            elif hasattr(exc, 'status_code') and exc.status_code == 429:
+                is_rate_limit = True
+            elif "rate limit" in str(exc).lower() or "429" in str(exc):
+                is_rate_limit = True
+            
+            # Try to extract Retry-After header from exception
+            if is_rate_limit:
+                try:
+                    if hasattr(exc, 'response') and hasattr(exc.response, 'headers'):
+                        retry_after_str = exc.response.headers.get('Retry-After', '')
+                        if retry_after_str:
+                            retry_after = float(retry_after_str)
+                except Exception:
+                    pass
+            
+            # Use Retry-After if available, otherwise use exponential backoff
+            if is_rate_limit and retry_after:
+                actual_sleep = retry_after
+                logger.warning(f"[retry {attempt + 1}/{retries}] Rate limit hit, waiting {actual_sleep}s per Retry-After header: {exc}")
+            elif is_rate_limit:
+                # For rate limits, use shorter initial backoff
+                actual_sleep = max(1.0, sleep_time * (1.5 ** attempt))
+                logger.warning(f"[retry {attempt + 1}/{retries}] Rate limit hit, retrying in {actual_sleep}s: {exc}")
+            else:
+                actual_sleep = sleep_time
+                logger.warning(f"[retry {attempt + 1}/{retries}] {exc}. Sleeping {actual_sleep}s.")
+            
+            time.sleep(actual_sleep)
 
 
-def _get_openai_embeddings(texts: List[str], *, model: str = "openai/text-embedding-3-large", batch_size: int = 100, max_workers: int = 10) -> List[List[float]]:
+def _get_openai_embeddings(texts: List[str], *, model: str = "openai/text-embedding-3-large", batch_size: int = 100, max_workers: int = 64) -> List[List[float]]:
     """Get embeddings for *texts* from the OpenAI API whilst preserving order."""
 
     if not texts:
@@ -190,6 +225,16 @@ def _get_embeddings(texts: List[str], embedding_model: str, verbose: bool = Fals
     if embedding_model == "openai" or str(embedding_model).startswith("openai/") or embedding_model in {"text-embedding-3-large", "text-embedding-3-small", "e3-large", "e3-small"}:
         return _get_openai_embeddings(texts, model=_normalize_embedding_model_name(embedding_model))
 
+    # Lazy import of sentence-transformers (optional dependency)
+    try:
+        from sentence_transformers import SentenceTransformer  # type: ignore
+    except ImportError:
+        raise ImportError(
+            "sentence-transformers is required for local embedding models. "
+            "Install it with: pip install stringsight[local-embeddings] "
+            "or: pip install sentence-transformers"
+        )
+
     if verbose:
         device = "cuda" if use_gpu else "cpu"
         logger.info(f"Computing embeddings with {embedding_model} on {device}…")
@@ -322,7 +367,7 @@ def assign_fine_to_coarse(
     model: str = "gpt-4.1-mini",
     strategy: str = "llm",
     verbose: bool = True,
-    max_workers: int = 10,
+    max_workers: int = 64,
 ) -> Dict[str, str]:
     """Assign each fine cluster name to one of the coarse cluster names.
 
@@ -398,7 +443,7 @@ def match_label_names(label_name, label_options):
             return option
     return None
 
-def llm_match(cluster_names, coarse_cluster_names, max_workers=10, model="gpt-4.1-mini"):
+def llm_match(cluster_names, coarse_cluster_names, max_workers=16, model="gpt-4.1-mini"):
     """Match fine-grained cluster names to coarse-grained cluster names using an LLM with parallel processing."""
     coarse_names_text = "\n".join(coarse_cluster_names)
     
@@ -473,6 +518,15 @@ def _setup_embeddings(texts, embedding_model, verbose=False, use_gpu=False):
             logger.debug(f"[emb-debug] normalized embeddings contain NaN/Inf: nan={nan_cnt} inf={inf_cnt}")
         return embeddings, None
     else:
+        # Lazy import of sentence-transformers (optional dependency)
+        try:
+            from sentence_transformers import SentenceTransformer  # type: ignore
+        except ImportError:
+            raise ImportError(
+                "sentence-transformers is required for local embedding models. "
+                "Install it with: pip install stringsight[local-embeddings] "
+                "or: pip install sentence-transformers"
+            )
         if verbose:
             device = "cuda" if use_gpu else "cpu"
             logger.info(f"Using sentence transformer: {embedding_model} on {device}")
@@ -503,7 +557,7 @@ def _get_openai_embeddings_batch_litellm(batch, retries=3, sleep_time=2.0):
 
 
 # NOTE: renamed to avoid overriding the DiskCache-cached version defined earlier
-def _get_openai_embeddings_litellm(texts, batch_size=100, max_workers=10):
+def _get_openai_embeddings_litellm(texts, batch_size=100, max_workers=16):
     """Get embeddings using OpenAI API (LiteLLM cache)."""
 
     if not texts:
