@@ -19,6 +19,7 @@ import random
 import os
 import pickle
 import argparse
+from tqdm import tqdm
 from ..logging_config import get_logger
 
 logger = get_logger(__name__)
@@ -42,7 +43,7 @@ from .config import ClusterConfig
 from .clustering_prompts import coarse_clustering_systems_prompt, deduplication_clustering_systems_prompt, outlier_clustering_systems_prompt
 
 # Optional imports (will be checked when needed)
-from sentence_transformers import SentenceTransformer
+# sentence-transformers is optional - imported lazily when needed
 import hdbscan
 import umap
 import litellm
@@ -167,9 +168,11 @@ def prepare_embeddings(unique_values: List[Any], config: ClusterConfig) -> np.nd
                             min_dist=config.umap_min_dist,
                             metric=config.umap_metric,
                             random_state=42,
-                            verbose=config.verbose
+                            verbose=False  # Disable UMAP's own verbose to use tqdm
                         )
-                        embeddings_reduced = reducer.fit_transform(embeddings_gpu)
+                        with tqdm(total=1, desc="UMAP dimensionality reduction (GPU)", disable=not config.verbose) as pbar:
+                            embeddings_reduced = reducer.fit_transform(embeddings_gpu)
+                            pbar.update(1)
                         # Convert back to numpy
                         embeddings = cp.asnumpy(embeddings_reduced)
                     except ImportError as e:
@@ -181,9 +184,11 @@ def prepare_embeddings(unique_values: List[Any], config: ClusterConfig) -> np.nd
                             min_dist=config.umap_min_dist,
                             metric=config.umap_metric,
                             random_state=42,
-                            verbose=config.verbose
+                            verbose=False  # Disable UMAP's own verbose to use tqdm
                         )
-                        embeddings = reducer.fit_transform(embeddings)
+                        with tqdm(total=1, desc="UMAP dimensionality reduction (CPU)", disable=not config.verbose) as pbar:
+                            embeddings = reducer.fit_transform(embeddings)
+                            pbar.update(1)
                 else:
                     reducer = umap.UMAP(
                         n_components=n_components,
@@ -191,9 +196,11 @@ def prepare_embeddings(unique_values: List[Any], config: ClusterConfig) -> np.nd
                         min_dist=config.umap_min_dist,
                         metric=config.umap_metric,
                         random_state=42,
-                        verbose=config.verbose
+                        verbose=False  # Disable UMAP's own verbose to use tqdm
                     )
-                    embeddings = reducer.fit_transform(embeddings)
+                    with tqdm(total=1, desc="UMAP dimensionality reduction (CPU)", disable=not config.verbose) as pbar:
+                        embeddings = reducer.fit_transform(embeddings)
+                        pbar.update(1)
                 
                 if config.verbose:
                     logger.info(f"Reduced to shape: {embeddings.shape}")
@@ -260,7 +267,7 @@ def generate_cluster_summaries(cluster_values: Dict[int, List], config: ClusterC
         messages,
         model=config.summary_model,
         system_prompt=clustering_systems_prompt,
-        max_workers=getattr(config, 'llm_max_workers', 10),
+        max_workers=getattr(config, 'llm_max_workers', 64),
         show_progress=config.verbose,
         progress_desc=f"Generating {cluster_type} summaries"
     )
@@ -406,7 +413,9 @@ def hdbscan_cluster_categories(df, column_name, config=None, **kwargs) -> pd.Dat
                     cluster_selection_method='eom',
                     cluster_selection_epsilon=config.cluster_selection_epsilon
                 )
-                cluster_labels_gpu = clusterer.fit_predict(embeddings_gpu)
+                with tqdm(total=1, desc=f"HDBSCAN clustering (GPU) - {len(embeddings)} items", disable=not config.verbose) as pbar:
+                    cluster_labels_gpu = clusterer.fit_predict(embeddings_gpu)
+                    pbar.update(1)
                 # Convert back to numpy
                 cluster_labels = cp.asnumpy(cluster_labels_gpu).astype(int)
             except ImportError as e:
@@ -422,7 +431,9 @@ def hdbscan_cluster_categories(df, column_name, config=None, **kwargs) -> pd.Dat
                     core_dist_n_jobs=-1,
                     cluster_selection_epsilon=config.cluster_selection_epsilon
                 )
-                cluster_labels = clusterer.fit_predict(embeddings)
+                with tqdm(total=1, desc=f"HDBSCAN clustering (CPU) - {len(embeddings)} items", disable=not config.verbose) as pbar:
+                    cluster_labels = clusterer.fit_predict(embeddings)
+                    pbar.update(1)
         else:
             clusterer = hdbscan.HDBSCAN(
                 min_cluster_size=effective_min_cluster_size,
@@ -434,7 +445,9 @@ def hdbscan_cluster_categories(df, column_name, config=None, **kwargs) -> pd.Dat
                 core_dist_n_jobs=-1,
                 cluster_selection_epsilon=config.cluster_selection_epsilon
             )
-            cluster_labels = clusterer.fit_predict(embeddings)
+            with tqdm(total=1, desc=f"HDBSCAN clustering (CPU) - {len(embeddings)} items", disable=not config.verbose) as pbar:
+                cluster_labels = clusterer.fit_predict(embeddings)
+                pbar.update(1)
 
     if config.verbose:
         n_initial_clusters = len(set(cluster_labels)) - (1 if -1 in cluster_labels else 0)
@@ -447,7 +460,6 @@ def hdbscan_cluster_categories(df, column_name, config=None, **kwargs) -> pd.Dat
             logger.info("Assigning outliers using BERTopic.reduce_outliers...")
         # Use OpenAIBackend if openai model, else None
         if config.embedding_model == "openai":
-            from bertopic.backend import OpenAIBackend
             import openai
             client = openai.OpenAI()
             bertopic_embedding_model = OpenAIBackend(client, "text-embedding-3-large")
@@ -489,8 +501,8 @@ def hdbscan_cluster_categories(df, column_name, config=None, **kwargs) -> pd.Dat
     # Get outlier items
     outlier_items = [unique_values[i] for i, label in enumerate(cluster_labels) if label < 0]
 
-    # Skip LLM-based outlier clustering if fewer than 5 outliers (not worth the overhead)
-    if len(outlier_items) >= 5:
+    # Skip LLM-based outlier clustering if fewer than min_cluster_size * 2 outliers (not worth the overhead)
+    if len(outlier_items) >= effective_min_cluster_size * 2:
         # Generate outlier cluster summaries
         outlier_cluster_names = generate_coarse_labels(
             outlier_items,
@@ -507,7 +519,7 @@ def hdbscan_cluster_categories(df, column_name, config=None, **kwargs) -> pd.Dat
             model=config.cluster_assignment_model,
             strategy="llm",
             verbose=config.verbose,
-            max_workers=getattr(config, 'llm_max_workers', 10),
+            max_workers=getattr(config, 'llm_max_workers', 64),
         )
         
         if config.verbose:
@@ -554,7 +566,7 @@ def hdbscan_cluster_categories(df, column_name, config=None, **kwargs) -> pd.Dat
             logger.info(f"Assigned outliers to {n_outlier_clusters} clusters, {n_remaining_outliers} remain as outliers")
     else:
         if config.verbose:
-            logger.info("No outliers to cluster")
+            logger.info(f"No outliers/too few outliers ({len(outlier_items)}) to cluster")
 
     # -------------------------------------------------------------
     # Step 4b: Deduplicate cluster labels via LLM            
@@ -584,7 +596,7 @@ def hdbscan_cluster_categories(df, column_name, config=None, **kwargs) -> pd.Dat
             model=config.cluster_assignment_model,
             strategy="llm",
             verbose=config.verbose,
-            max_workers=getattr(config, 'llm_max_workers', 10),
+            max_workers=getattr(config, 'llm_max_workers', 64),
         )
 
         # -------------------------------------------------------------
