@@ -23,11 +23,97 @@ from .caching import Cache
 logger = logging.getLogger(__name__)
 
 
+# ==================== LiteLLM Timing Callback ====================
+try:
+    from litellm.integrations.custom_logger import CustomLogger as LiteLLMCustomLogger
+except ImportError:
+    LiteLLMCustomLogger = object
+
+
+class LLMTimingCallback(LiteLLMCustomLogger):
+    """Custom LiteLLM callback to log API call timing information."""
+
+    def log_pre_api_call(self, model, messages, kwargs):
+        """Called before API call - not used for timing."""
+        pass
+
+    def log_post_api_call(self, kwargs, response_obj, start_time, end_time):
+        """Log timing for every API call."""
+        try:
+            duration = (end_time - start_time).total_seconds()
+            model = kwargs.get("model", "unknown")
+            print(f"[LLM Timing] {model}: {duration:.3f}s", flush=True)
+        except Exception:
+            pass
+
+    def log_success_event(self, kwargs, response_obj, start_time, end_time):
+        """Log successful API calls with detailed timing."""
+        try:
+            duration = (end_time - start_time).total_seconds()
+            model = kwargs.get("model", "unknown")
+
+            # Extract token usage if available
+            usage = getattr(response_obj, 'usage', None)
+            if usage:
+                prompt_tokens = getattr(usage, 'prompt_tokens', 0)
+                completion_tokens = getattr(usage, 'completion_tokens', 0)
+                total_tokens = getattr(usage, 'total_tokens', 0)
+                print(
+                    f"[LLM Success] {model}: {duration:.3f}s | "
+                    f"tokens: {total_tokens} (prompt: {prompt_tokens}, completion: {completion_tokens})",
+                    flush=True
+                )
+            else:
+                print(f"[LLM Success] {model}: {duration:.3f}s", flush=True)
+        except Exception:
+            pass
+
+    def log_failure_event(self, kwargs, response_obj, start_time, end_time):
+        """Log failed API calls."""
+        try:
+            duration = (end_time - start_time).total_seconds()
+            model = kwargs.get("model", "unknown")
+            error = kwargs.get("exception", "unknown error")
+            print(f"[LLM Failure] {model}: {duration:.3f}s | error: {error}", flush=True)
+        except Exception:
+            pass
+
+
+# Global timing callback instance
+_timing_callback = None
+
+
+def enable_timing_logs():
+    """Enable detailed timing logs for all LLM API calls."""
+    global _timing_callback
+    if _timing_callback is None:
+        _timing_callback = LLMTimingCallback()
+        if not hasattr(litellm, 'callbacks') or litellm.callbacks is None:
+            litellm.callbacks = []
+        litellm.callbacks.append(_timing_callback)
+        logger.info("✓ LLM timing logs enabled")
+
+
+def disable_timing_logs():
+    """Disable detailed timing logs for LLM API calls."""
+    global _timing_callback
+    if _timing_callback is not None and hasattr(litellm, 'callbacks'):
+        if litellm.callbacks and _timing_callback in litellm.callbacks:
+            litellm.callbacks.remove(_timing_callback)
+        _timing_callback = None
+        logger.info("✓ LLM timing logs disabled")
+
+
+# Enable timing logs by default
+# enable_timing_logs()  # Disabled by default - call manually to enable
+# ================================================================
+
+
 @dataclass
 class LLMConfig:
     """Configuration for LLM calls."""
     model: str = "gpt-4o-mini"
-    max_workers: int = 10
+    max_workers: int = 64
     max_retries: int = 3
     base_sleep_time: float = 2.0
     timeout: Optional[float] = None
@@ -41,7 +127,7 @@ class EmbeddingConfig:
     """Configuration for embedding calls."""
     model: str = "text-embedding-3-small"
     batch_size: int = 100
-    max_workers: int = 10
+    max_workers: int = 64
     max_retries: int = 3
     base_sleep_time: float = 2.0
 
@@ -60,7 +146,8 @@ class LLMUtils:
         config: LLMConfig,
         system_prompt: Optional[str] = None,
         show_progress: bool = True,
-        progress_desc: str = "LLM calls"
+        progress_desc: str = "LLM calls",
+        progress_callback: Optional[Callable[[int, int], None]] = None
     ) -> List[str]:
         """
         Execute LLM completions in parallel with order preservation.
@@ -71,7 +158,8 @@ class LLMUtils:
             system_prompt: Optional system prompt (if messages are strings)
             show_progress: Whether to show progress bar
             progress_desc: Description for progress bar
-            
+            progress_callback: Optional callback(completed, total) called after each completion
+
         Returns:
             List of completion responses in the same order as input
         """
@@ -98,9 +186,13 @@ class LLMUtils:
                     request_data = {
                         "model": config.model,
                         "messages": request_messages,
-                        "temperature": config.temperature,
-                        "top_p": config.top_p,
                     }
+                    
+                    # GPT-5 models only support temperature=1, so skip temperature/top_p for them
+                    if "gpt-5" not in config.model.lower():
+                        request_data["temperature"] = config.temperature
+                        request_data["top_p"] = config.top_p
+                    
                     if config.max_tokens:
                         request_data["max_completion_tokens"] = config.max_tokens
                     
@@ -119,24 +211,36 @@ class LLMUtils:
                             caching=False,  # Use our own caching
                             timeout=config.timeout
                         )
-                    except litellm.BadRequestError as api_err:
-                        # Some models only accept default temperature/top_p; drop unsupported args and retry once
+                    except Exception as api_err:
+                        # Check if this is an unsupported params error
                         err_text = str(api_err).lower()
-                        safe_request = dict(request_data)
-                        retried = False
-                        if "temperature" in err_text:
-                            safe_request.pop("temperature", None)
-                            retried = True
-                        if "top_p" in err_text:
-                            safe_request.pop("top_p", None)
-                            retried = True
-                        if retried:
-                            response = litellm.completion(
-                                **safe_request,
-                                caching=False,
-                                timeout=config.timeout
-                            )
+                        err_type = type(api_err).__name__.lower()
+                        
+                        # Handle unsupported parameter errors (BadRequestError, UnsupportedParamsError, etc.)
+                        if ("unsupported" in err_text or "don't support" in err_text or 
+                            "badrequest" in err_type or "unsupportedparams" in err_type):
+                            safe_request = dict(request_data)
+                            retried = False
+                            
+                            if "temperature" in err_text:
+                                safe_request.pop("temperature", None)
+                                retried = True
+                                logger.info(f"Dropping temperature parameter for {config.model}")
+                            if "top_p" in err_text:
+                                safe_request.pop("top_p", None)
+                                retried = True
+                                logger.info(f"Dropping top_p parameter for {config.model}")
+                            
+                            if retried:
+                                response = litellm.completion(
+                                    **safe_request,
+                                    caching=False,
+                                    timeout=config.timeout
+                                )
+                            else:
+                                raise
                         else:
+                            # Not an unsupported params error, re-raise to outer handler
                             raise
                     
                     content = response.choices[0].message.content
@@ -158,8 +262,41 @@ class LLMUtils:
                         logger.error(f"LLM call failed after {config.max_retries} attempts: {e}")
                         return idx, f"ERROR: {e}"
                     else:
-                        sleep_time = config.base_sleep_time * (2 ** attempt)
-                        logger.warning(f"LLM call attempt {attempt + 1} failed, retrying in {sleep_time}s: {e}")
+                        # Check if this is a rate limit error
+                        is_rate_limit = False
+                        retry_after = None
+                        
+                        # Check for litellm rate limit errors
+                        if hasattr(litellm, 'RateLimitError') and isinstance(e, litellm.RateLimitError):
+                            is_rate_limit = True
+                        elif hasattr(e, 'status_code') and e.status_code == 429:
+                            is_rate_limit = True
+                        elif "rate limit" in str(e).lower() or "429" in str(e):
+                            is_rate_limit = True
+                        
+                        # Try to extract Retry-After header from exception
+                        if is_rate_limit:
+                            try:
+                                if hasattr(e, 'response') and hasattr(e.response, 'headers'):
+                                    retry_after_str = e.response.headers.get('Retry-After', '')
+                                    if retry_after_str:
+                                        retry_after = float(retry_after_str)
+                            except Exception:
+                                pass
+                        
+                        # Use Retry-After if available, otherwise use exponential backoff
+                        if is_rate_limit and retry_after:
+                            sleep_time = retry_after
+                            logger.warning(f"Rate limit hit (attempt {attempt + 1}/{config.max_retries}), waiting {sleep_time}s per Retry-After header: {e}")
+                        elif is_rate_limit:
+                            # For rate limits, use shorter initial backoff but still exponential
+                            sleep_time = max(1.0, config.base_sleep_time * (1.5 ** attempt))
+                            logger.warning(f"Rate limit hit (attempt {attempt + 1}/{config.max_retries}), retrying in {sleep_time}s: {e}")
+                        else:
+                            # For other errors, use standard exponential backoff
+                            sleep_time = config.base_sleep_time * (2 ** attempt)
+                            logger.warning(f"LLM call attempt {attempt + 1} failed, retrying in {sleep_time}s: {e}")
+                        
                         time.sleep(sleep_time)
             
             return idx, "ERROR: Max retries exceeded"
@@ -167,18 +304,23 @@ class LLMUtils:
         # Execute in parallel
         with ThreadPoolExecutor(max_workers=config.max_workers) as executor:
             futures = {
-                executor.submit(_single_completion, idx, msg): idx 
+                executor.submit(_single_completion, idx, msg): idx
                 for idx, msg in enumerate(messages)
             }
-            
+
             # Process results with optional progress bar
-            iterator = as_completed(futures)
-            if show_progress:
-                iterator = tqdm(iterator, total=len(messages), desc=progress_desc)
-                
-            for future in iterator:
+            # Use manual progress updates for smoother per-item tracking
+            pbar = tqdm(total=len(messages), desc=progress_desc, disable=not show_progress)
+            completed = 0
+            for future in as_completed(futures):
                 idx, result = future.result()
                 results[idx] = result
+                completed += 1
+                pbar.update(1)
+                # Call progress callback if provided
+                if progress_callback:
+                    progress_callback(completed, len(messages))
+            pbar.close()
         
         return results
     
@@ -187,7 +329,7 @@ class LLMUtils:
         texts: List[str],
         config: EmbeddingConfig,
         show_progress: bool = True,
-        progress_desc: str = "Embedding calls"
+        progress_desc: str = "Generating embeddings"
     ) -> List[List[float]]:
         """
         Generate embeddings in parallel with batching and order preservation.
@@ -268,8 +410,41 @@ class LLMUtils:
                         fallback_embeddings = [[0.0] * 1536] * len(batch_texts)  # Default to 1536 dims
                         return start_idx, fallback_embeddings
                     else:
-                        sleep_time = config.base_sleep_time * (2 ** attempt)
-                        logger.warning(f"Embedding batch attempt {attempt + 1} failed, retrying in {sleep_time}s: {e}")
+                        # Check if this is a rate limit error
+                        is_rate_limit = False
+                        retry_after = None
+                        
+                        # Check for litellm rate limit errors
+                        if hasattr(litellm, 'RateLimitError') and isinstance(e, litellm.RateLimitError):
+                            is_rate_limit = True
+                        elif hasattr(e, 'status_code') and e.status_code == 429:
+                            is_rate_limit = True
+                        elif "rate limit" in str(e).lower() or "429" in str(e):
+                            is_rate_limit = True
+                        
+                        # Try to extract Retry-After header from exception
+                        if is_rate_limit:
+                            try:
+                                if hasattr(e, 'response') and hasattr(e.response, 'headers'):
+                                    retry_after_str = e.response.headers.get('Retry-After', '')
+                                    if retry_after_str:
+                                        retry_after = float(retry_after_str)
+                            except Exception:
+                                pass
+                        
+                        # Use Retry-After if available, otherwise use exponential backoff
+                        if is_rate_limit and retry_after:
+                            sleep_time = retry_after
+                            logger.warning(f"Rate limit hit (attempt {attempt + 1}/{config.max_retries}), waiting {sleep_time}s per Retry-After header: {e}")
+                        elif is_rate_limit:
+                            # For rate limits, use shorter initial backoff but still exponential
+                            sleep_time = max(1.0, config.base_sleep_time * (1.5 ** attempt))
+                            logger.warning(f"Rate limit hit (attempt {attempt + 1}/{config.max_retries}), retrying in {sleep_time}s: {e}")
+                        else:
+                            # For other errors, use standard exponential backoff
+                            sleep_time = config.base_sleep_time * (2 ** attempt)
+                            logger.warning(f"Embedding batch attempt {attempt + 1} failed, retrying in {sleep_time}s: {e}")
+                        
                         time.sleep(sleep_time)
             
             # Fallback
@@ -282,16 +457,16 @@ class LLMUtils:
                 executor.submit(_single_embedding_batch, start, batch_texts): (start, len(batch_texts))
                 for start, batch_texts in batches
             }
-            
+
             # Process results with optional progress bar
-            iterator = as_completed(futures)
-            if show_progress:
-                iterator = tqdm(iterator, total=len(batches), desc=progress_desc)
-            
-            for future in iterator:
+            # Track by number of texts embedded, not batches, for better granularity
+            pbar = tqdm(total=len(texts), desc=progress_desc, disable=not show_progress)
+            for future in as_completed(futures):
                 start_idx, batch_embeddings = future.result()
                 batch_size = len(batch_embeddings)
                 embeddings[start_idx:start_idx + batch_size] = batch_embeddings
+                pbar.update(batch_size)
+            pbar.close()
         
         # Verify all embeddings were filled
         if any(e is None for e in embeddings):
@@ -349,28 +524,29 @@ def parallel_completions(
     messages: List[Union[str, List[Dict[str, Any]]]],
     model: str = "gpt-4o-mini",
     system_prompt: Optional[str] = None,
-    max_workers: int = 10,
+    max_workers: int = 64,
     show_progress: bool = True,
     progress_desc: str = "LLM calls",
+    progress_callback: Optional[Callable[[int, int], None]] = None,
     **kwargs
 ) -> List[str]:
     """Convenience function for parallel completions with default settings."""
     # Separate function-specific parameters from config parameters
-    config_kwargs = {k: v for k, v in kwargs.items() 
+    config_kwargs = {k: v for k, v in kwargs.items()
                     if k in ['max_retries', 'base_sleep_time', 'timeout', 'temperature', 'top_p', 'max_tokens']}
-    
+
     config = LLMConfig(model=model, max_workers=max_workers, **config_kwargs)
     utils = get_default_llm_utils()
-    return utils.parallel_completions(messages, config, system_prompt, show_progress, progress_desc)
+    return utils.parallel_completions(messages, config, system_prompt, show_progress, progress_desc, progress_callback)
 
 
 def parallel_embeddings(
     texts: List[str],
     model: str = "text-embedding-3-small",
     batch_size: int = 100,
-    max_workers: int = 10,
+    max_workers: int = 64,
     show_progress: bool = True,
-    progress_desc: str = "Embedding calls",
+    progress_desc: str = "Generating embeddings",
     **kwargs
 ) -> List[List[float]]:
     """Convenience function for parallel embeddings with default settings."""
