@@ -9,6 +9,7 @@ import uuid
 from typing import Dict, Any, Optional, List
 from pathlib import Path
 import pandas as pd
+from tqdm import tqdm
 from ..core.stage import PipelineStage
 from ..core.data_objects import PropertyDataset, Property
 from ..core.mixins import LoggingMixin, TimingMixin, ErrorHandlingMixin, WandbMixin
@@ -52,8 +53,9 @@ class LLMJsonParser(LoggingMixin, TimingMixin, ErrorHandlingMixin, WandbMixin, P
         empty_list_responses = 0  # Track when LLM returns empty lists
         consecutive_errors = 0  # Track consecutive parsing errors
         max_consecutive_errors = 10
-        
-        for i, prop in enumerate(data.properties):
+
+        # Add progress bar for better visibility
+        for i, prop in enumerate(tqdm(data.properties, desc="Parsing properties", disable=not getattr(self, 'verbose', False))):
             # We only process properties that still have raw_response
             if not prop.raw_response:
                 # Debug: Print information about the property with empty raw_response
